 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c MultiWell: a code for chemical kinetics calculations.
c Copyright (C) 2013 John R. Barker
c
c John R. Barker
c jrbarker@umich.edu
c University of Michigan
c Ann Arbor, MI 48109-2143
c (734) 763 6239
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License (version 2)
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
c GNU General Public License for more details.
c
c See the 'ReadMe' file for a copy of the GNU General Public License,
c or contact:
c
c Free Software Foundation, Inc.
c 59 Temple Place - Suite 330
c Boston, MA 02111-1307, USA.
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c    k(E) calculated and stored in double array 'Rate'.  Note that
c    reverse rxns are calculated with same sums of states as
c    forward rxns.
c
c    T    = translational temperature
c    Mol  = index of reactant molecule
c    Nchan     = number of channels originating from Mol
c    Jread     = 0  use inverse Laplace transform method for channel
c         = 1  read sums of states for channel from external file
c         = 2  read k(E) for this channel from external file
c         = 3  read cumulative rxn prob (crp) for channel from external file
c    Jto  = index of product molecule
c    Jrev = 0 for neglecting the reverse reaction
c         = 1 for calculating reverse rxn rate; after calculation,
c              Jrev=1 is changed to Jrev=2
c    Afact     = A-factor for reaction (units: s-1) [for ILT method]
c    Eact = activation energy (units: cm-1) [for ILT method]
c
c    Note that File name and path in OPEN statements may need revision
c         for non-Macintosh
c
c***********************************************************************
c       KEYWORDS:
c
c         'NOCENT'   no centrifugal correction             [NCENT =0]
c         'CENT1'    1-D adiabatic rotor centrifugal       [NCENT= 1]
c         'CENT2'    2-D adiabatic rotor centrifugal       [NCENT= 2]
c         'CENT3'    3-D adiabatic rotor centrifugal       [NCENT= 3]
c         'CENTX'    LEGACY centrifugal correction         [NCENT=-2]
c                    (not recommended)
c
c         'NOREV'    neglecting the reverse reaction        [Jrev = Jr]
c         'REV'      calculating reverse rxn rate; 
c                    afterwards, Jrev=1 is changed to Jrev=2
c
c         'FAST'     neglecting limitations due to IVR
c         'SLOW'     including IVR limitations; a following line contains 
c                        'SLOW', pivr(Mol,i), vivr(Mol,i), vave(Mol,i), 
c                             tivr(Mol,i), civr(Mol,i,1), civr(Mol,i,2), 
c                             civr(Mol,i,3)
c
c         'NOTUN'    for neglecting tunneling
c         'TUN'      for including tunneling via unsymmetrical Eckart barrier;
c                       a following line contains: 'TUN', tunnel(Mol,i)
c
c          'ILT'     Inverse laplace transform method for k(E)
c          'SUM'     File containing sums of states (usually generated by Densum)
c          'RKE'     File containing k(E), generated externally
c          'REVRKE'  k(E) for reverse reaction calculated from k(E) file for forward rxn
c***********************************************************************
c         WARNINGS:
c          WARN1    = warning that k(E) = 0 was imposed because Product Well has empty grain corresponding to E
c***********************************************************************
      SUBROUTINE RateArray(T)
      IMPLICIT NONE
      INCLUDE 'declare1.inc'
      INTEGER i , k , Mol , JJ , jdum , lenstr
      INTEGER J , Jmax , KE
      DOUBLE PRECISION Bmol , Bts , EJmol , delEJ, delEJmax , Qmol , fac
      DOUBLE PRECISION arg , X , Y , Y1 , Y2 , Y3
      DOUBLE PRECISION DENVR , E , EE , DD , A , B , T , DUM , 
     &                 ER , Test1 , Test2 , cfac , f , rat , Eto , Dto
      DOUBLE PRECISION psum , absbar , statest
      CHARACTER TITLE*100
      SAVE 

      statest = 1.e-05/Egrain1	! statest: Density criterion to test if a state is present in a given energy bin 

      DO Mol = 1 , NWells
            DO i = 1 , Nchan(Mol)
 
               Path(Mol,i) = (Molsym(Mol)*TSele(Mol,i)*TSopt(Mol,i))
     &                       /(TSsym(Mol,i)*Molele(Mol)*Molopt(Mol))
                          ! Rxn path degeneracy X electronic partition fxn ratio used for all k(E)
                          
c              ! ++++++++++++++STANDARD CENTRIFUGAL CORRECTION++++++++++++
c
c                'NOCENT'   no centrifugal correction             [NCENT=  0]
c                'CENT1'    1-D adiabatic rotor centrifugal       [NCENT=  1]
c                'CENT2'    2-D adiabatic rotor centrifugal       [NCENT=  2]
c                'CENT3'    2-D adiabatic rotor centrifugal       [NCENT=  3]
c                'CENTX'    LEGACY centrifugal correction         [NCENT= -2]  (not recommended)

               IF (     (MolMom(Mol) .GT. 0.0 )
     &            .AND. (TSmom(Mol,i) .GT. 0.0)  )  THEN
                  rat = TSmom(Mol,i)/MolMom(Mol)
                 ELSE
                  rat = 1.0D+00
               ENDIF
               IF ( NCENT(Mol,i) .EQ. 0 ) THEN
                  cfac = 1.0d+00
                  Eor(Mol,i) = Eo(Mol,i)
               ELSEIF ( NCENT(Mol,i).GT.0 .AND. NCENT(Mol,i).LE.3) THEN 
                  f = 0.5*NCENT(Mol,i)*( 1.0d+00 - rat )      ! takes into account the number of adiabatic rotors
                  Eor(Mol,i) = Eo(Mol,i) + f*T/hor
                  cfac = rat*EXP(f)
                  IF ( NCENT(Mol,i) .EQ. -2 ) cfac = 1.d+00
               ENDIF
               IF ( ( Eor(Mol,i) .LT. 0.0 ) .AND. 
     &              ( Jread(Mol,i) .EQ. 1 )  ) THEN
                 write(*,*) ' '
                 write(*,*) '********** WARNING **********'
                 write(*,*) ' Eocent should not be <0 for SUM input '
                 write(*,*) ' Centrifugal correction is too large.'
                 write(*,*) '   Reaction from ....... to '
                 write(*,*) '   ', Mol, Jto(Mol,i)
                 write(*,*) '********** WARNING **********'
                 write(KOUT,*) ' '
                 write(KOUT,*) '********** WARNING **********'
                 write(KOUT,*) ' Eocent should not be <0 for SUM input '
                 write(KOUT,*) ' Centrifugal correction is too large'
                 write(KOUT,*) '   Reaction from ....... to '  
                 write(KOUT,*) '   ', Mol, Jto(Mol,i)
                 write(KOUT,*) '********** WARNING **********'
                 write(KSMM,*) ' '
                 write(KSMM,*) '********** WARNING **********'
                 write(KSMM,*) ' Eocent should not be <0 for SUM input '
                 write(KSMM,*) ' Centrifugal correction is too large'
                 write(KSMM,*) '   Reaction from ....... to '   
                 write(KSMM,*) '   ', Mol, Jto(Mol,i)
                 write(KSMM,*) '********** WARNING **********'
                 write(KSMM,*) ' '
               ENDIF                                    ! ++++++++++++++END STANDARD CENTRIFUGAL CORRECTION++++++++++++

               IF ( Jread(Mol,i).EQ.0 ) THEN            ! ++++++++++++++Inverse-Laplace-Transform++++++++++++++
                   
                  DO 5 k = 1 , Isize
                     IF ( k.LE.imax1 ) THEN             ! E = energy from top of barrier
                        E = (k-1)*Egrain1
                     ELSE
                        E = (k-imax1-1)*Egrain2
                     ENDIF
                     EE = E + Eo(Mol,i)                 ! Does NOT include centrifugal correction
                     DD = DENVR(Mol,EE)
                     IF ( EE .LE. Emax2 .AND. DD.GT.statest ) THEN   ! calculate k(E) only up to Emax2, since dens is undefined for higher E
                       Rate(Mol,i,k) = Afac(Mol,i)*DENVR(Mol,E)/DD    ! k(E)
                       Rate(Mol,i,k) = LOG(Rate(Mol,i,k))             ! k(E) stored as log
                     ELSE
                       Rate(Mol,i,k) = LOG( 1.e-20 )                  ! k(E) stored as log
                     ENDIF
                       
 5                CONTINUE
 
               ELSEIF ( Jread(Mol,i).EQ.1 ) THEN        ! ++++++++++++++Sums of states from external file++++++++++++++
                  OPEN (12,FILE='DensData/'//
     &            TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.dens',
     &            STATUS='OLD')                         ! TST sum of states

                  READ (12,99030) dumcut                ! Check to see if file format includes input data summary
                  IF ( dumcut .EQ. cut ) THEN           ! Format starting with v.2008.1
                    dumcut = 'x'
                    DO WHILE (dumcut .NE. cut)
                      READ (12,99030) dumcut
                    END DO
                 ELSE
                    BACKSPACE 12                        ! If format from prior to v.2008.1
                 ENDIF
99030 FORMAT (A46)
                 READ (12,*) TITLE
                 READ (12,*) TITLE
                 READ (12 , * )  A , JJ , B , k , DUM
                 READ (12,*) TITLE
                 Test1 = ABS(A-Egrain1)/1.D-06
                 Test2 = ABS(B-Emax2)/1.D-06
                 IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. ! Test for proper array bounds, etc.
     &              Test2.LT.1.0 .AND. k.EQ.Isize ) THEN

	               absbar=HMol(Mol)+Eor(Mol,i)   ! absolute energy of barrier needed to determine
                                                  ! back reaction barrier

                    V1(Mol,i)=absbar-HMol(Jto(Mol,i))   ! HMol(Jto(Mol,i)) is the enthalpy of the product


                    if(itun(Mol,i).eq.1)then     ! check to make sure the back reaction barrier is
                                                 ! greater than zero, which must be the case if tunneling
                                                 ! is to be considered
                      if(V1(Mol,i).lt.0)then
                         write(6,99005)Jto(Mol,i),Mol
                         itun(Mol,i)=0
                      end if
                    end if

                    if(itun(Mol,i) .eq. 0) then    ! ++++++++++++++ NO tunneling section ++++++++++++++

                       DO k = 1 , Isize
                         READ (12,*) jdum , A , B , TSsum(Mol,i,k)    ! Read sum of states  (here, A and B are dummy variables)
                       END DO  ! k
                       
                    else                        ! ++++++++++++++ TUNNELING section *************************************

                      write(*,99003) Mol        ! Entering tunneling routine write statement

                      CALL DensArrayTS(Mol,i)   ! set up the densTS array so that the density of states
                                                ! of the transition state can be interpolated later
                      do k=1,Isize+imax1        ! loop over energies for which k(E) will be calculated

                        IF ( k.LE.imax1*2 ) THEN                
                          E = (k-1)*Egrain1               
                          EE=Eor(Mol,i)-((imax1-1)*Egrain1)+E  ! Includes centrifugal correction
                        ELSE 
                          E = (k-imax1*2-1)*Egrain2
                          EE=Eor(Mol,i)+E  ! Includes centrifugal correction; ensures 2nd part
                        ENDIF              ! of double array starts at 0 cm-1 instead of below the barrier


                        if((Jrev(Mol,Jto(Mol,i))).eq.1)then  ! if the reverse rxn from well Mol to product Jto(Mol,i)
                                                             ! is considered than store the density of states of the 
                                                             ! product calculated for the forward rxn in TSdensBK  and 
                                                             ! use this quantity to calculate the reverse rxn rate 

                           iset(Mol)=Jto(Mol,i)           ! the reverse rxn will have iset(Mol) as the well and 
                           iset2(Mol)=Mol                 ! iset2(Mol) as the product

                        else if((Mol.eq.iset(Mol)).and.
     &                        (Jto(Mol,i).eq.iset2(Mol)))then

                           if(TSdensBK(iset2(Mol),k).lt.1e-30)then
                              TRate(Mol,i,k)=-70.0d0       
                              goto 19                      
                           end if

                           if(DENVR(Mol,EE).lt.statest)then
                              TRate(Mol,i,k)=-70.0d0
                              goto 19
                           end if

                           IF ( EE .LE. Emax2 ) THEN                  ! If molecular energy is not too high, claculate k(E)
                              TRate(Mol,i,k)=TSdensBK(iset2(Mol),k)*
     &                               clight*Path(Mol,i)/DENVR(Mol,EE) !    k(E) with tunneling correction
                              TRate(Mol,i,k) = LOG(TRate(Mol,i,k))    !    k(E) with tunneling correction stored as log
                              goto 19
                           ELSE
                              TRate(Mol,i,k) = -70.0                  ! Do not calculate k(E) if molecular energy >Emax2
                           ENDIF

                        end if 

                            
                        CALL ECKART(Mol,i,EE,psum,k)  ! calculate tunneling correction using 
                                                       ! unsymmetrical Eckart potential


                        if((Jrev(Mol,Jto(Mol,i))).eq.1)then   ! if the reverse rxn  Jto(Mol,i)-->Mol is considered 
                           TSdensBK(Mol,k)=psum               ! save the denisty of states of the TS for the forward
                        end if                                ! rxn and use them to calculate the reverse rxn rate, i.e.
                                                              ! don't perform the calculation twice

                        if(psum.lt.1e-30)then         ! this if statement is necessary because if the sum of 
                           TRate(Mol,i,k)=-70.0d0     ! is zero taking LOG(TRate(Mol,i,k) below results in NAN 
                           goto 19                    ! this circumvents that problem
                        end if

                        if(DENVR(Mol,EE).lt.statest)then
                          TRate(Mol,i,k)=-70.0d0
                          goto 19
                        end if

                        IF ( EE .LE. Emax2 ) THEN                    ! If molecular energy is not too high, claculate k(E)
                           TRate(Mol,i,k) = psum*clight*Path(Mol,i)
     &                                       /DENVR(Mol,EE)          ! k(E) with tunneling correction
                           TRate(Mol,i,k) = LOG(TRate(Mol,i,k))      ! k(E) with tunneling correction stored as log
                        ELSE
                           TRate(Mol,i,k) = -70.0                    ! Do not calculate k(E) if molecular energy >Emax2
                        ENDIF

19                         CONTINUE
                      end do  ! k

                           write(*,99004)
                           
                    end if   ! no-tunneling/tunneling section
                    
                  ELSE                               ! If grain or energy limit does not match
                    WRITE (KSTD,99002) TSname(Mol,i)
                    WRITE (KOUT,99002) TSname(Mol,i)
                    STOP
                  ENDIF
                  CLOSE (12)
 
               ELSEIF ( Jread(Mol,i).EQ.3 ) THEN        ! ++++++++++++++Read CRP file++++++++++++++
                 OPEN (12,FILE='DensData/'//
     &           TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.crp',
     &           STATUS='OLD')                          ! CRP computed using SCTST of states
                  READ (12,99030) dumcut                ! Check to see if file format includes input data summary
                  IF ( dumcut .EQ. cut ) THEN           ! Format starting with v.2008.1
                    dumcut = 'x'
                    DO WHILE (dumcut .NE. cut)
                      READ (12,99030) dumcut            ! Check to see if file format includes input data summary
                    END DO
                 ELSE
                    BACKSPACE 12                        ! If format from prior to v.2008.1
                 ENDIF
                 READ (12,*) TITLE
                 READ (12,*) TITLE
                 READ (12 , * )  A , JJ , B , k , DUM
                 READ (12,*) TITLE
                 Test1 = ABS(A-Egrain1)/1.D-06
                 Test2 = ABS(B-Emax2)/1.D-06
                 IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. ! Test for proper array bounds, etc.
     &              Test2.LT.1.0 .AND. k.EQ.Isize ) THEN

                       DO k = 1 , Isize
                         READ (12,*) jdum , A , B , TSsum(Mol,i,k)   ! Read sum of states  (here, A and B are dummy variables)
                       END DO   ! k
                       
                 ENDIF
                 CLOSE (12)
                 

               ELSEIF ( Jread(Mol,i).EQ.2 ) THEN        ! ++++++++++++++Read k(E)'s from external file ++++++++++++++ (not including path degeneracy) 
                  OPEN (12,FILE='DensData/'//
     &            TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.rke',
     &            STATUS='OLD')                   ! k(E)'s
                  READ (12,*) TITLE
                  READ (12,*) TITLE
c
c                           Egrain1 , imax1 , Emax2 , Isize
                  READ (12 , * )  A , JJ , B , k , DUM
                  READ (12,*) TITLE
                  Test1 = ABS(A-Egrain1)/1.D-06
                  Test2 = ABS(B-Emax2)/1.D-06
                  IF ( Test1.LT.1.0 .AND. JJ.EQ.imax1 .AND. 
     &                 Test2.LT.1.0 .AND. k.EQ.Isize ) THEN     ! Test for proper array bounds, etc.
                     DO k = 1 , Isize
                        READ (12,*) jdum , A , B , Rate(Mol,i,k)    ! Read k(E)'s
                        IF ( Afac(Mol,i).lt.0 ) Rate(Mol,i,k) =
     &                    Rate(Mol,i,k)*ABS(Afac(Mol,i))
                        IF ( Rate(Mol,i,k).GT.1.D-20 ) THEN
                           Rate(Mol,i,k) = LOG(Rate(Mol,i,k))       ! k(E) stored as log
                        ELSE
                           Rate(Mol,i,k) = LOG(1.D-20)              ! k(E) stored as log
                        ENDIF
                     END DO   ! k
                  ELSE                  ! If grain or energy limit does not match
                     WRITE (KSTD,99002) TSname(Mol,i)
                     WRITE (KOUT,99002) TSname(Mol,i)
                     STOP
                  ENDIF
                  CLOSE (12)
 
               ELSEIF ( Jread(Mol,i).EQ.-2 ) THEN       ! ++++++++++++++Read k(E)'s for REVERSE rxn (not including path degeneracy) ++++++++++++++
                  OPEN (12,FILE='DensData/'//
     &               TSname(Mol,i)(1:lenstr(TSname(Mol,i)))//'.rke',
     &               STATUS='OLD')                                    ! open k(E)'s previously computed for what is now the REVERSE rxn
                  READ (12,*) TITLE
                  READ (12,*) TITLE
c
c                           Egrain1 , imax1 , Emax2 , Isize
                  READ (12 , * )  A , JJ , B , k , DUM
                  READ (12,*) TITLE
                  Test1 = ABS(A-Egrain1)/1.D-06
                  Test2 = ABS(B-Emax2)/1.D-06
                  IF ( Test1 .LT. 1.0 .AND. JJ .EQ. imax1 .AND. 
     &                 Test2 .LT .1.0 .AND. k .EQ. Isize ) THEN
                                                ! Test for proper array bounds, etc.
                     DO k = 1 , Isize
                        READ (12,*) jdum , A , B , Rate(Mol,i,k)  ! Read k(E)'s for REVERSE reaction (not including path degeneracy) and use them to fill this array for the forward rxn
 
                        IF ( k.LE.imax1 ) THEN                  ! E = energy from top of barrier
                           E = (k-1)*Egrain1                    ! Energy in forward direction
                        ELSE
                           E = (k-imax1-1)*Egrain2              ! Energy in forward direction
                        ENDIF
                        E = E + Eor(Mol,i)                      ! With centrifugal correction
                        ER = E - (HMol(Jto(Mol,i))-HMol(Mol))   ! Energy in reverse direction
                        DD = DENVR(Jto(Mol,i),ER)/DENVR(Mol,E)  ! Density of states ratio
                        Rate(Mol,i,k) = Rate(Mol,i,k)*DD*Path(Mol,i)
                                                                ! now use DD and Path degeneracy to obtain k(E) in forward direction
                        IF ( Rate(Mol,i,k).GT.1.D-20 ) THEN
                           Rate(Mol,i,k) = LOG(Rate(Mol,i,k))
                                                        ! k(E) stored as log
                        ELSE
                           Rate(Mol,i,k) = LOG(1.D-20)  ! k(E) approximately =0 stored as log
                        ENDIF

                     END DO  ! k
                  ELSE                  ! If grain or energy limit does not match
                     WRITE (KSTD,99002) TSname(Mol,i)
                     WRITE (KOUT,99002) TSname(Mol,i)
                     STOP
                  ENDIF
                  CLOSE (12)
 
               ENDIF
               
               IF ( Jread(Mol,i).EQ.1 .OR. Jread(Mol,i).EQ.3 ) THEN      ! ++++++++ COMPUTE RATE CONSTANTS ++++++++
c              Jread     = 0  use inverse Laplace transform method for channel
c                        = 1  read sums of states for channel from external file
c                        = 2  read k(E) for this channel from external file
c                        = 3  read cumulative rxn prob from crp file
c                                                                        ! +++++++++IF STANDARD CENTRIFUGAL CORRECTION+++++
c                'NOCENT'   no centrifugal correction             [NCENT=  0]
c                'CENT1'    1-D adiabatic rotor centrifugal       [NCENT=  1]
c                'CENT2'    2-D adiabatic rotor centrifugal       [NCENT=  2]
c                'CENT3'    3-D adiabatic rotor centrifugal       [NCENT=  3]
c                'CENTX'    LEGACY centrifugal correction         [NCENT= -2]  (not recommended)

                   DO k = 1 , Isize
                     IF ( k.LE.imax1 ) THEN                    ! E = active energy in TS from top of barrier+ZPE
                       E = (k-1)*Egrain1                       ! Active energy in Transition State
                     ELSE
                       E = (k-imax1-1)*Egrain2                 ! Active energy in Transition State
                     ENDIF
                     EE = E + Eor(Mol,i)                       ! Active energy in REACTANT Well, includes centrifugal correction

                     IF ( EE.LE.Emax2 ) THEN                   ! calculate k(E) only up to Emax2, since Wells only have states up to Emax2
                       DD = DENVR(Mol,EE)                              ! density of states in reactant Well at energy EE
                       IF ( DD.GE.statest) THEN         
                         DD = TSsum(Mol,i,k)/DD 
                         Rate(Mol,i,k) = clight*DD*Path(Mol,i)     ! k(E) w/o tunneling
                         Rate(Mol,i,k) = cfac*Rate(Mol,i,k)        ! ++++++++ STANDARD CENTRIFUGAL CORRECTION +++++++
                         Rate(Mol,i,k) = LOG(Rate(Mol,i,k)+1.e-20) ! k(E) w/o tunneling correction stored as log
                       ELSE
                         Rate(Mol,i,k)= LOG( 1.d-20 )   ! If no state in this grain (empty grain),  rate = 0.
                       ENDIF
                     ELSE
                       Rate(Mol,i,k)= LOG( 1.d-20 )     ! if EE .GE. Emax2
                     ENDIF
                   END DO   ! k
                 ENDIF  ! Jread

c    FOR WELL-TO-WELL RXNS, CHECK TO MAKE SURE A STATE IS PRESENT IN THE PRODUCT WELL, regardless of how k(E) was calculated.

               WARN1(Mol,i) = 0                                   ! For counting empty grains (that contain no states)
               IF ( Jto(Mol,i) .LE. NWells ) THEN
                   DO k = 1 , Isize
                     IF ( k.LE.imax1 ) THEN                       ! E = energy in TS from top of barrier+ZPE
                       E = (k-1)*Egrain1                          ! energy in Transition State
                     ELSE
                       E = (k-imax1-1)*Egrain2                    ! energy in Transition State
                     ENDIF
                     EE = E + Eor(Mol,i)                          ! energy in REACTANT Well, includes possible centrifugal correction
                     Eto = EE + HMol(Mol) - HMol(Jto(Mol,i))      ! energy of PRODUCT WELL

                     IF ( Eto.GE.0.d+00 .AND. EE.GE.0.d+00 ) THEN
                        IF ( EE.LE.Emax2 .AND. Eto.LE.Emax2 ) THEN      ! calculate k(E) only up to Emax2, since Wells only have states up to Emax2
                          DD = DENVR(Mol,EE)                            ! density of states in reactant Well at energy EE
                          IF ( DD.GE.statest) THEN         
                             Dto = DENVR( Jto(Mol,i), Eto )             ! Test to see if a state exists in PRODUCT WELL energy grain
                             IF ( Dto .LT. statest ) THEN
                               WARN1(Mol,i) = WARN1(Mol,i) + 1          ! count number of empty grains
                               Rate(Mol,i,k)= LOG( 1.d-21 )             ! IF NO STATE IS PRESENT in this PRODUCT grain, then set k(E) = 0
                             ENDIF
                          ENDIF
                        ELSE
                          Rate(Mol,i,k)= LOG( 1.d-22 )
                        ENDIF
                     ELSE
                        WARN1(Mol,i) = WARN1(Mol,i) + 1            ! count number of empty grains
                        Rate(Mol,i,k)= LOG( 1.d-23 )               ! IF Eto < 0 or EE < 0, then set k(E) = 0
                     ENDIF
                   END DO   ! k
               ENDIF        ! end of test for empty grains

           END DO  ! index i (rxn channels)
        END DO  ! index Mol (loop over Wells)
         
        RETURN
 
99001   FORMAT (A100)
99002   FORMAT (/'Energy grain or energy limit does not match: ',
     &        A10/'****** Run Terminated *******')
99003   FORMAT ('Entering Tunneling Subroutine: Molecule #',I2)
99004   FORMAT ('Leaving Tunneling Subroutine')
99005   FORMAT (/'The reaction barrier from product',i3,' to well', i3,
     &          /' is less than the enthalpy of this product.',
     &   /'Calculations will proceed as if tunneling were not invoked.')
 
        END

